'
' 256 point FFT routines, using S:2:13 Fixed Point Format for numeric computations
' All data is stored in the the Word Array "FFT_Data".
'
	Include "FFT Macros.Inc"					' Load the macros into the program
	Include "Misc Subroutines.Inc"				' Load the General Purpose Subroutines into memory

'--------------------------------------------------------------------------------------------------
' Do a Complex FFT, radix-1, decimation in place, 128 points
' Input 	: 128 complex values (4 bytes), stored in FFT_Data[0..255]
'          		For each data element:
'					Byte 0 = LSB of Real part
'                   Byte 1 = MSB of Real part
'                  	Byte 2 = LSB of Complex part
'                   Byte 3 = MSB of Complex part
'
' Output	: Replace FFT_Data[0..255] by its FFT (128 frequencies, Complex Values)
' Notes		: Algorithm adapted from the book "Numerical recipes in C"
'
PerformFFT Macro
	GoSub FFTSub
	Endm
FFTSub:
'
' Bit Reversal
'
	j = 0
	i = 0
	Repeat
		If j > i Then
			AccumA = FFT_Data[i]			' Save the contents of FFT_Data[i]
			FFT_Data[i] = FFT_Data[j]
			FFT_Data[j] = AccumA
			AccumA = FFT_Data[i + 1]		' Save the contents of FFT_Data[i + 1]
			FFT_Data[i + 1] = FFT_Data[j + 1]
			FFT_Data[j + 1] = AccumA
		EndIf
		m = n / 2
		While 1 = 1
			If m < 2 Then Break
			If j < m Then Break
			j = j - m
			m = m / 2
		Wend
		j = j + m
		i = i + 2
	Until OverFlowFlag = 1 				' A Loop of 0 to 255 i.e. n - 1
'
' The Actual FFT routine
'
  	mmax = 2
	While 1 = 1
		If mmax = 0 Then
			wpr = $C008
			wpi = $FFFF
			Break
		EndIf
		istep = mmax * 2

		' More Efficient equivalent of: Theta = 2 * pi / mmax
		AccumA = Fixed_PI					' 3.14159265459
		DivTemp = mmax / 2
		GoSub FixedDivide
		Theta = AccumA

		' More Efficient equivalent of: wpr = -2 * Sin(Theta / 2) ^ 2
		AccumA = Div2 [AccumA]
		GoSub FixedSin
		AccumB = FixedMult [AccumA, AccumA]
		wpr = FixedMult [AccumB, $C000]		' AccumB * -2.0

		' More Efficient equivalent of: wpi = Sin(Theta)
		AccumA = Theta
		GoSub FixedSin
		wpi = AccumA
		wr = $2000 ' wr = 1.0
		wi = $0000 						' wi = 0.0
		m = 0
		Repeat
			i = m
Indexloop:
			j = i + mmax
			'
			' Pre-Capture values from the FFT_Data array to speed up the routine
			'
			DataSaveJ = FFT_Data[j]
			DataSaveJ1 = FFT_Data[j + 1]
			DataSaveI = FFT_Data[i]
			DataSaveI1 = FFT_Data[i + 1]

			' More Efficient equivalent of: Temp1r = wr * FFT_Data[j] - wi * FFT_Data[j + 1]
			If wi <> 0 Then					' Save an unnecessary multiply
				AccumA = FixedMult [wr, DataSaveJ]
				Temp1r = FixedMult [wi, DataSaveJ1]
				Temp1r = AccumA - Temp1r
			Else
				Temp1r = FixedMult [wr, DataSaveJ]
			EndIf

			' More Efficient equivalent of: Temp1i = wr * FFT_Data[j + 1] + wi * FFT_Data[j]
			If wi <> 0 Then					' Save an unnecessary multiply
				Temp1i = FixedMult [wi, DataSaveJ]
				AccumA = FixedMult [wr, DataSaveJ1]
				Temp1i = Temp1i + AccumA
			Else
				Temp1i = FixedMult [wr, DataSaveJ1]
			EndIf
			'
			' Butterfly update with Scaling
			'
			' More Efficient equivalent of: FFT_Data[j] = (FFT_Data[i] - Temp1r) / 2
			AccumA = DataSaveI - Temp1r
			AccumA = Div2 [AccumA]
			FFT_Data[j] = AccumA

			' More Efficient equivalent of: FFT_Data[j + 1] = (FFT_Data[i + 1] - Temp1i) / 2
			AccumA = DataSaveI1 - Temp1i
			AccumA = Div2 [AccumA]
			FFT_Data[j + 1] = AccumA

			' More Efficient equivalent of: FFT_Data[i] = (FFT_Data[i] + Temp1r) / 2
			AccumA = DataSaveI + Temp1r
			AccumA = Div2 [AccumA]
			FFT_Data[i] = AccumA

			' More Efficient equivalent of: FFT_Data[i + 1] = (FFT_Data[i + 1] + Temp1i) / 2
			AccumA = DataSaveI1 + Temp1i
			AccumA = Div2 [AccumA]
			FFT_Data[i + 1] = AccumA

			If istep <> 0 Then
				i = i + istep
				If OverFlowFlag = 0 Then GoTo Indexloop
			EndIf
			AccumB = wr						' Save the contents of wr

			' More Efficient equivalent of: wr = wr * wpr - wi * wpi + wr
			Temp1r = FixedMult [wi, wpi]
			AccumA = FixedMult [wr, wpr]
			AccumA = AccumA - Temp1r
			wr = wr + AccumA

 			' More Efficient equivalent of: wi = wi * wpr + AccumB * wpi + wi
			Temp1i = FixedMult [AccumB, wpi]
			AccumA = FixedMult [wi, wpr]
			AccumA = AccumA + Temp1i
			wi = wi + AccumA
			m = m + 2
		Until m >= mmax - 1
		mmax = istep
	Wend
'
' Find the Values from the above FFT result
'
	Theta = $00C9 					    ' Theta = 2 * pi / n
	wpr = $FFFE			 				' wpr = -2 * Sin(Theta / 2) ^ 2
	wpi = $00C8			 				' wpi = Sin(2 * pi / n)
	wr = $1FFE			  				' wr = wpr + 1.0
	wi = wpi
	i = 2
	Repeat
		Int1 = i + i
		Int1 = Int1 - 2
		Int2 = Int1 + 1
		Int3 = -Int2
		Int3 = Int3 + 1
		Int4 = Int3 + 1
		'
		' Pre-Capture values from the FFT_Data array to speed up the routine
		'
		DataSave1 = FFT_Data[Int1]
		DataSave2 = FFT_Data[Int2]
		DataSave3 = FFT_Data[Int3]
		DataSave4 = FFT_Data[Int4]

		' More Efficient equivalent of: Temp1r = (FFT_Data[Int1] + FFT_Data[Int3]) / 2
		Temp1r = DataSave1 + DataSave3
		Temp1r = Div2 [Temp1r]

		' More Efficient equivalent of: Temp1i = (FFT_Data[Int2] - FFT_Data[Int4]) / 2
		Temp1i = DataSave2 - DataSave4
		Temp1i = Div2 [Temp1i]

		' More Efficient equivalent of: Temp2r = (FFT_Data[Int2] + FFT_Data[Int4]) / 2
		Temp2r = DataSave2 + DataSave4
		Temp2r = Div2 [Temp2r]

		' More Efficient equivalent of: Temp2i = -(FFT_Data[Int1] - FFT_Data[Int3]) / 2
		Temp2i = DataSave3 - DataSave1
		Temp2i = Div2 [Temp2i]
		'
		' Pre-Multiply key variables to streamline the FFT
		'
		wi_X_Temp2i = FixedMult [wi, Temp2i]
		wr_X_Temp2r = FixedMult [wr, Temp2r]
		wr_X_Temp2i = FixedMult [wr, Temp2i]
		wi_X_Temp2r = FixedMult [wi, Temp2r]

		' More Efficient equivalent of: FFT_Data[Int1] = Temp1r + wr * Temp2r - wi * Temp2i
		AccumA = wr_X_Temp2r - wi_X_Temp2i
		AccumA = AccumA + Temp1r
		FFT_Data[Int1] = AccumA

		' More Efficient equivalent of: FFT_Data[Int2] = Temp1i + wr * Temp2i + wi * Temp2r
		AccumA = wr_X_Temp2i + wi_X_Temp2r
		AccumA = AccumA + Temp1i
		FFT_Data[Int2] = AccumA

		' More Efficient equivalent of: FFT_Data[Int3] = Temp1r - wr * Temp2r + wi * Temp2i
		AccumA = wi_X_Temp2i - wr_X_Temp2r
		AccumA = AccumA + Temp1r
		FFT_Data[Int3] = AccumA

		' More Efficient equivalent of: FFT_Data[Int4] = -Temp1i + wr * Temp2i + wi * Temp2r
		AccumA = wi_X_Temp2r + wr_X_Temp2i
		AccumA = AccumA - Temp1i
		FFT_Data[Int4] = AccumA
		AccumB = wr						' Save the value of wr

		' More Efficient equivalent of: wr = wr * wpr - wi * wpi + wr
		Temp1r = FixedMult [wi, wpi]
		AccumA = FixedMult [wr, wpr]
		AccumA = AccumA - Temp1r
		wr = wr + AccumA

		' More Efficient equivalent of: wi = wi * wpr + AccumB * wpi + wi
		Temp1r = FixedMult [wi, wpr]
		AccumA = FixedMult [AccumB, wpi]
		AccumA = AccumA + Temp1r
		wi = wi + AccumA
		Inc i
	Until i > (n / 4)
'
' Special case of the first two data elements
'
	AccumA = FFT_Data#0						' Save the value of FFT_Data[0]
	FFT_Data#0 = FFT_Data#0 + FFT_Data#1
	FFT_Data#1 = AccumA - FFT_Data#1
	Return


