'
' 256 Point (128 point result) FFT Routine using Fixed Point Calculations
'
' Accepts an audio signal on analogue input pin AN0 (PORTA.0)
' Displays the spectrum on a Samsung KS0108 Graphic LCD
'
' The circuit for this demo can be found in the same folder as this program
' and is named "Fixed Point FFT Circuit and Layout.pdf"

Warnings = Off
Reminders = Off
	Optimiser_Level = 3									' Maximum Optimisation
'
' Setup the pins for the Samsung KS0108 Graphic chipset on the LCD shield
'   
    LCD_CS1Pin = PORTC.0                                ' Graphic LCD's CS1 pin
    LCD_CS2Pin = PORTC.1                                ' Graphic LCD's CS2 pin
    LCD_RSPin = PORTA.3                                 ' Graphic LCD's RS pin
    LCD_RWPin = PORTA.5                                 ' Graphic LCD's RW pin
    LCD_ENPin = PORTA.4                                 ' Graphic LCD's EN pin
    LCD_DTPort = PORTB                                  ' Graphic LCD's Data Port  
    LCD_Type = Samsung                                  ' We're using a Samsung KS0108 graphic LCD
    Internal_Font = On                                  ' An internal (code memory) font

	Unsigned_Dwords = On								' We're not using signed Dwords

'--------------------------------------------------------------------------------------------------
    Include "ADC.inc"                                   ' Load the ADC macros into the program
	Include "Variable Definitions.Inc"					' Load the Variable and Constant Definitions into the program

'--------------------------------------------------------------------------------------------------

	GoTo Main									         ' Jump Over the FFT Subroutines

'--------------------------------------------------------------------------------------------------

	Include "FFT Subroutines.Inc"						' Load the FFT Subroutines into memory

'--------------------------------------------------------------------------------------------------
' Capture the audio input
' And fill the FFT_Data Array with the scaled ADC Samples
'
AquireADC:
	NumberOfSamples = 0
	Repeat
		ADCON0bits_GO_DONE = 1
		While ADCON0bits_GO_DONE = 1 : Wend				' Wait for a 10-bit Left Justified (Fractional) ADC value
		ADCResult = ADCResult >> 2
		ADCResult.HighByte = ADCResult.HighByte - $20   ' Subtract $2000 (fixed point 1.0) So it's now from -1.0 to +1.0
		FFT_Data[NumberOfSamples] = ADCResult			' Place the value into the array
		DelayUS 100										' Alter this value for lesser or greater capture speeds
		Inc NumberOfSamples
	Until OverFlowFlag = 1								' A loop of 256
	Return
'--------------------------------------------------------------------------------------------------
' The Main Program Starts here
'
' Notes		: In order to display the S:2:13 fixed point value, use a variation of the code:
'			:	Rlcf AccumAH,w : TempWord = AccumA : If STATUS.0 = 1 Then TempWord = -TempWord : Hrsout "-" 	' Check and display the Sign Bit
'			: 	TempFloat = TempWord / 8192 : Hrsout Dec3 TempFloat,13 											' Convert to a Float and Display
'			: Where AccumA holds the Fixed Point Value
'
Main:
	Clear										' Clear all RAM before we start
'
' Open the ADC:
'               Fosc Set For Fosc / 32
'               Left justified for fractional operation
'               Tad value of 2
'               Vref+ at Vcc : Vref- at Gnd
'               Make AN0 an analogue input
'
    OpenADC(ADC_FOSC_32 & ADC_LEFT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_1ANA)

	Cls											' Clear the LCD
'
' Perform the FFT and display the spectrum on the Graphic LCD
'
	While 1 = 1
		GoSub AquireADC												' Fill the FFT array with ADC readings
		PerformFFT													' Perform the FFT on the data received
		CalculateLog												' Scale the result using a Logarithmic scale (Remark if not used)
		'CalculateLin												' Scale the result using a Linear scale (Remark if not used)
		'
		' Display the spectrum on the graphic LCD
		' Alter this routine as required.
		'
		NumberOfSamples = 127										' Ignore the first sample (DC Content)
		RealIndex = 1
		Repeat
			TempByte = RealBuffer[RealIndex]						' Read a value from the array
			If TempByte >= 63 Then TempByte = 63
			PreviousRealValue = ShadowRealBuffer[RealIndex]			' Read a value from the shadow array
			If PreviousRealValue <> TempByte Then
				Line 0,NumberOfSamples,0,NumberOfSamples, PreviousRealValue
			EndIf
			If TempByte > 1 Then
				Line 1,NumberOfSamples,0,NumberOfSamples, TempByte
			EndIf
			ShadowRealBuffer[RealIndex] = TempByte					' Keep a record of the line for erasure
			Inc RealIndex
			Dec NumberOfSamples
		Until NumberOfSamples = 0									' Perform a loop of 128
	Wend
'--------------------------------------------------------------------------------------------------

